@@ -239,8 +239,6 @@ class Calculator {
 	time_t exchange_rates_time, exchange_rates_check_time;
 	bool b_exchange_rates_used, b_exchange_rates_warning_enabled;
 
-	int has_gvfs;
-
 	bool b_gnuplot_open;
 	string gnuplot_cmdline;
 	FILE *gnuplot_pipe;
@@ -265,12 +263,12 @@ class Calculator {
 
 	vector<MathStructure*> rpn_stack;
 
-	bool calculateRPN(MathStructure *mstruct, int command, size_t index, int msecs, const EvaluationOptions &eo);
-	bool calculateRPN(string str, int command, size_t index, int msecs, const EvaluationOptions &eo, MathStructure *parsed_struct, MathStructure *to_struct, bool make_to_division);
+	bool calculateRPN(MathStructure *mstruct, int command, size_t index, int msecs, const EvaluationOptions &eo, int function_arguments = 0);
+	bool calculateRPN(string str, int command, size_t index, int msecs, const EvaluationOptions &eo, MathStructure *parsed_struct, MathStructure *to_struct, bool make_to_division, int function_arguments = 0);
 	
   public:
 
-	KnownVariable *v_pi, *v_e, *v_i, *v_inf, *v_pinf, *v_minf, *v_undef;
+	KnownVariable *v_pi, *v_e, *v_euler, *v_catalan, *v_i, *v_inf, *v_pinf, *v_minf, *v_undef, *v_precision;
 	UnknownVariable *v_x, *v_y, *v_z;
 	MathFunction *f_vector, *f_sort, *f_rank, *f_limits, *f_component, *f_dimension, *f_merge_vectors;
 	MathFunction *f_matrix, *f_matrix_to_vector, *f_area, *f_rows, *f_columns, *f_row, *f_column, *f_elements, *f_element, *f_transpose, *f_identity, *f_determinant, *f_permanent, *f_adjoint, *f_cofactor, *f_inverse; 
@@ -279,12 +277,12 @@ class Calculator {
 	MathFunction *f_abs, *f_gcd, *f_lcm, *f_signum, *f_round, *f_floor, *f_ceil, *f_trunc, *f_int, *f_frac, *f_rem, *f_mod;	
 	MathFunction *f_polynomial_unit, *f_polynomial_primpart, *f_polynomial_content, *f_coeff, *f_lcoeff, *f_tcoeff, *f_degree, *f_ldegree;
 	MathFunction *f_re, *f_im, *f_arg, *f_numerator, *f_denominator;
-  	MathFunction *f_sqrt, *f_sq;
+  	MathFunction *f_sqrt, *f_cbrt, *f_root, *f_sq;
 	MathFunction *f_exp;
 	MathFunction *f_ln, *f_logn;
 	MathFunction *f_lambert_w;
-	MathFunction *f_sin, *f_cos, *f_tan, *f_asin, *f_acos, *f_atan, *f_sinh, *f_cosh, *f_tanh, *f_asinh, *f_acosh, *f_atanh, *f_radians_to_default_angle_unit;
-	MathFunction *f_zeta, *f_gamma, *f_beta;
+	MathFunction *f_sin, *f_cos, *f_tan, *f_asin, *f_acos, *f_atan, *f_sinh, *f_cosh, *f_tanh, *f_asinh, *f_acosh, *f_atanh, *f_atan2, *f_radians_to_default_angle_unit;
+	MathFunction *f_zeta, *f_gamma, *f_digamma, *f_beta, *f_airy, *f_besselj, *f_bessely, *f_erf, *f_erfc;
 	MathFunction *f_total, *f_percentile, *f_min, *f_max, *f_mode, *f_rand;
 	MathFunction *f_isodate, *f_localdate, *f_timestamp, *f_stamptodate, *f_days, *f_yearfrac, *f_week, *f_weekday, *f_month, *f_day, *f_year, *f_yearday, *f_time, *f_add_days, *f_add_months, *f_add_years;
 	MathFunction *f_bin, *f_oct, *f_hex, *f_base, *f_roman;
@@ -314,6 +312,7 @@ class Calculator {
 	MathStructure *tmp_rpn_mstruct;
 	bool tmp_maketodivision;
 	int tmp_proc_command;
+	int tmp_proc_registers;
 	size_t tmp_rpnindex;
 	
 	PrintOptions save_printoptions;	
@@ -358,6 +357,17 @@ class Calculator {
 	*/
 	MathStructure calculate(string str, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
 	/** Calculates a parsed value.
+	* This function starts the calculation in a separate thread and will return when the calculation has started unless a maximum time has been specified.
+	* The calculation can then be stopped with abort().
+	*
+	* @param[out] mstruct Parsed value to evaluate and fill with the result.
+	* @param msecs The maximum time for the calculation in milliseconds. If msecs <= 0 the time will be unlimited.
+	* @param eo Options for the evaluation of the expression.
+	* @param to_str "to" expression for conversion.
+	* @returns The result of the calculation.
+	*/
+	bool calculate(MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_evaluation_options, string to_str = "");
+	/** Calculates a parsed value.
 	*
 	* @param mstruct Parsed value to evaluate.
 	* @param eo Options for the evaluation of the expression.
@@ -395,17 +405,17 @@ class Calculator {
 	string abortedMessage(void) const;
 	string timedOutString(void) const;
 	
-	/** Deprecated: use startControl() */
+	///Deprecated: use startControl()
 	void startPrintControl(int milliseconds = 0);
-	/** Deprecated: use abort() */
+	///Deprecated: use abort()
 	void abortPrint(void);
-	/** Deprecated: use stopControl() */
+	///Deprecated: use stopControl()
 	void stopPrintControl(void);
-	/** Deprecated: use aborted() */
+	/// Deprecated: use aborted()
 	bool printingAborted(void);
-	/** Deprecated: use isControlled() */
+	/// Deprecated: use isControlled()
 	bool printingControlled(void) const;
-	/** Deprecated: use abortedMessage() */
+	/// Deprecated: use abortedMessage()
 	string printingAbortedMessage(void) const;
 	//@}
 
@@ -413,9 +423,9 @@ class Calculator {
 	//@{
 	/** Returns true if the calculate or print thread is busy. */
 	bool busy();
-	/** Deprecated: does nothing. */
+	/// Deprecated: does nothing.
 	void saveState();
-	/** Deprecated: does nothing. */
+	/// Deprecated: does nothing.
 	void restoreState();
 	/** Clears all stored values. Used internally after aborted calculation. */
 	void clearBuffers();
@@ -981,22 +991,14 @@ class Calculator {
 	//@}
 	
 	/** @name Functions for exchange rates. */
-	//@{
-	/** Checks if gvfs-copy or wget is available for downloading exchange rates from the Internet.
+	/** Checks if able to downloading exchange rates from the Internet (using libcurl).
 	*
-	* @returns true if gvfs-copy or wget was found.
+	* @returns true if exchange rates can downloaded (if libcurl is available).
 	*/
 	bool canFetch();
-	/** Checks if gvfs-copy available.
-	*
-	* @returns true if gvfs-copy was found.
-	*/
+	///Deprecated: gvfs is not needed anymore.
 	bool hasGVFS();
-	/** Checks if gvfs-copy available. Deprecated: use hasGVFS() or canFetch() instead.
-	*
-	*
-	* @returns true if gvfs-copy was found.
-	*/
+	///Deprecated: gvfs is not needed anymore.
 	bool hasGnomeVFS();
 	/** Load saved (local) currency units and exchange rates.
 	*
@@ -1008,26 +1010,23 @@ class Calculator {
 	* @returns name of local exchange rates file.
 	*/
 	string getExchangeRatesFileName();
+	string getExchangeRatesFileName2();
 	/** Url of the exchange rates file on the Internet.
 	*
 	* @returns Url of exchange rates file.
 	*/
 	string getExchangeRatesUrl();
+	string getExchangeRatesUrl2();
 	/** Modification time of the exchange rates file.
 	*
 	* @returns Returns exchange rates modification time.
 	*/
 	time_t getExchangeRatesTime();
-	/** Download current exchange rates from the Internet to local disc.
-	*
-	* @param milliseconds Maximum time for donwload try (only used by wget)
-	* @param wget_args Extra arguments to pass to wget.
-	* @returns true if operation was successful.
-	*/
+	///Deprecated: wget arguments are not used
 	bool fetchExchangeRates(int seconds, string wget_args);
 	/** Download current exchange rates from the Internet to local disc with default wget arguments.
 	*
-	* @param milliseconds Maximum time for donwload try (only used by wget)
+	* @param seconds Maximum time for donwload try
 	* @returns true if operation was successful.
 	*/
 	bool fetchExchangeRates(int seconds = 15);
@@ -1142,4 +1141,43 @@ class Calculator {
 		
 };
 
+class QalculateDate {
+	protected:
+		long int i_year;
+		long int i_month;
+		long int i_day;
+	public:
+		QalculateDate();
+		QalculateDate(long int initialyear, int initialmonth, int initialday);
+		QalculateDate(long int initialtimestamp);
+		QalculateDate(string date_string);
+		bool operator > (const QalculateDate &date2) const;
+		bool operator < (const QalculateDate &date2) const;
+		bool operator >= (const QalculateDate &date2) const;
+		bool operator <= (const QalculateDate &date2) const;
+		bool operator != (const QalculateDate &date2) const;
+		bool operator == (const QalculateDate &date2) const;
+		bool isFutureDate() const;
+		bool isPastDate() const;
+		void setToCurrentDate();
+		bool set(long int newyear, int newmonth, int newday);
+		bool set(long int newtimestamp);
+		bool set(string date_string);
+		string toISOString() const;
+		string toLocalString() const;
+		long int year() const;
+		long int month() const;
+		long int day() const;
+		bool addDays(long int days);
+		bool addMonths(long int months);
+		bool addYears(long int years);
+		int weekday() const;
+		int week(bool start_sunday = false) const;
+		int yearday() const;
+		Number timestamp() const;
+		Number daysTo(const QalculateDate &date, int basis = 1, bool date_func = true) const;
+		Number yearsTo(const QalculateDate &date, int basis = 1, bool date_func = true) const;
+};
+
+
 #endif
